/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                          MODULE TCPExample                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2021.04.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "tcpservice.h"
#include "trace.h"

#define IRQCallCntLimit                 100

#define MagicSygnature                  0xAA55

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 TCPLayerInstanceRecT *      TCPLayer ;
                 PtrTCPSocketDescriptionRecT TCPSocket ;
               } TCPSeverInstanceRecT ;
typedef TCPSeverInstanceRecT * PtrTCPSeverInstanceRecT ;

static TCPSeverInstanceRecT TCPSeverInstance ;
static ULONG IRQCallCounter ;

#define MachineIPAddress        0xC0A80037
//                              ^^^ 192.168.0.55
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254
#define TCPPortNumber           2000


UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;




static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrTCPSeverInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  UCHAR LocalMyMACAddr [ 6 ] ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 6 ; Loop ++ )
    LocalMyMACAddr [ Loop ] = MachineMACAddr [ Loop ] ;
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( LocalMyMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer , MachineIPAddress ,
                                              MachineIPSubnet ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = MachineGWIPAddress ;
  Instance -> TCPLayer = StandardAddTCPInstance ( Instance -> IPLayer ) ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: Serwer polaczen TCP\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello5 [ ]      = "\r\nNasluch na porcie TCP=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;
UCHAR Error1 [ ]      = "\r\nProblem z gniazdkiem\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceIP ( MachineIPAddress ) ;
  UART0SendString ( Hello3 ) ;
  TraceIP ( MachineIPSubnet ) ;
  UART0SendString ( Hello4 ) ;
  TraceIP ( MachineGWIPAddress ) ;
  UART0SendString ( Hello5 ) ;
  TraceUSHORTdec ( TCPPortNumber ) ;
  UART0SendString ( Hello6 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


USHORT TCPConnectOperation ( PtrTCPSocketDescriptionRecT TCPSocket ,
                             ULONG                       RemoteAddress ,
                             USHORT                      RemotePort )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( "Proba nawiazania polaczenia z IP=" ) ;
  TraceIP ( RemoteAddress ) ;
  UART0SendString ( " port=" ) ;
  TraceUSHORTdec ( RemotePort ) ;
  UART0SendString ( "\r\n" ) ;
  return ( TRUE ) ;
} /* TCPConnectOperation */


void TCPEstablishOperation ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( "Polaczenie zestawione\r\n" ) ;
  TCPSendData ( TCPSocket , "Witaj koles przy kompie\r\n\r\n" , 27 ) ;
} /* TCPEstablishOperation */

                                                                                
void TCPCloseOperation ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( "Polaczenie zamkniete\r\n" ) ;
} /* TCPCloseOperation */


void TCPRecvOperation ( PtrTCPSocketDescriptionRecT TCPSocket ,
                        ULONG                       SenderIPAddress ,
                        USHORT                      SenderPortNo ,
                        ULONG                       DestIPAddress ,
                        USHORT                      DestPortNo ,
                        void *                      DataBuffer ,
                        USHORT                      DataBufferSize )
{
  UCHAR * Buffer ;
  USHORT Loop ;
  UCHAR SendData [ 32 ] ;
  UCHAR * ChPtr ;
  /*-------------------------------------------------------------------------*/
  if ( DataBufferSize > 0 )
  {
    Buffer = ( UCHAR * ) DataBuffer ;
    UART0SendString ( "Przyszly dane: " ) ;
    for ( Loop = 0 ; Loop < DataBufferSize ; Loop ++ )
      UART0Send ( * ( Buffer  + Loop ) ) ;
    UART0Send ( '\r' ) ;
    UART0Send ( '\n' ) ;
    if ( DataBufferSize > 30 )
      DataBufferSize = 30 ;
    ChPtr = SendData ;
    * ChPtr ++ = '*' ;
    for ( Loop = 0 ; Loop < DataBufferSize ; Loop ++ )
      * ChPtr ++ = * ( Buffer  + Loop ) ;
    * ChPtr ++ = '*' ;
    TCPSendData ( TCPSocket , SendData , DataBufferSize + 2 ) ;
  } /* if */ ;
} /* TCPRecvOperation */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  TCPSeverInstance . EMACLayer = NIL ;
  TCPSeverInstance . IPLayer = NIL ;
  TCPSeverInstance . TCPLayer = NIL ;
  TCPSeverInstance . TCPSocket = NIL ;
} /* SoftwareInit */


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & TCPSeverInstance ) ;
  TCPSeverInstance . TCPSocket = CreateTCPSocket ( TCPSeverInstance . TCPLayer ) ;
  if ( BindTCPSocket ( TCPSeverInstance . TCPSocket , TCPPortNumber ) != TCP_BindPortOK )
  {
    Halt ( Error1 ) ;
  } /* if */ ;
  if ( TCPListen ( TCPSeverInstance . TCPSocket , TCPConnectOperation , TCPEstablishOperation ,
                   TCPRecvOperation , TCPCloseOperation ) != TCP_ListenPortOK )
  {
    Halt ( Error1 ) ;
  } /* if */ ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    TCPSeverInstance . IPLayer -> IPLayerPoolService ( TCPSeverInstance . IPLayer ) ;
  } /* loop */ ;
} /* main */

